/*
 * Copyright (C) 2005  Heinz Mauelshagen, Red Hat GmbH.
 *                     All rights reserved.
 *
 * See file LICENSE at the top of this source tree for license information.
 */

#include "internal.h"
#include "socket.h"
#include "interface.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

static struct daemon_params *alloc_and_init_params(void)
{
	struct daemon_params *ret;

	if (!(ret = dbg_malloc(sizeof(*ret))))
		return NULL;

	ret->fd   = -1;

	/* FIXME: must be configurable. */
	ret->host = (char*) DAEMON_HOST;
	ret->file = (char*) DAEMON_NAME;
	ret->port = DAEMON_PORT;

	return ret;
}	

static int daemon_open(struct lib_context *lc)
{
	struct hostent *he;
	struct sockaddr_in server;
	struct daemon_params *params = lc->daemon->private;

	if (params) {
		if (params->fd != -1)
			return 1;
	} else if (!(params = alloc_and_init_params()))
		return 0;

	if (!(he = gethostbyname(params->host))) {
		log_err(lc, "gethostbyname");
		goto free;
	}

	if ((params->fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		log_err(lc, "socket");
		goto free;
	}

	memset(&server, 0, sizeof(server));

	server.sin_family = AF_INET;
	server.sin_port   = htons(params->port);

	/* he->h_addr passes "*he"'s info to "h_addr". */
	server.sin_addr   = *((struct in_addr *)he->h_addr);

	if (connect(params->fd, (struct sockaddr*) &server,
		    sizeof(struct sockaddr)) == -1) {
		log_err(lc, "connect");
		goto close;
	}

	/* Keep the daemon communication parameters. */
	lc->daemon->private = params;

	return 1;

   close:
	close (params->fd);

   free:
	dbg_free(params);

	return 0;
}

static void daemon_close(struct lib_context *lc)
{
	if (lc->daemon->private) {
		close(((struct daemon_params*) lc->daemon->private)->fd);
		dbg_free(lc->daemon->private);
		lc->daemon->private = NULL;
	}
}

static int daemon_read(struct lib_context *lc, struct daemon_message *msg)
{
	return read(((struct daemon_params*) lc->daemon->private)->fd,
		    msg, sizeof(*msg)) == sizeof(*msg);
}

static int daemon_write(struct lib_context *lc, struct daemon_message *msg)
{
	return write(((struct daemon_params*) lc->daemon->private)->fd,
		     msg, sizeof(*msg)) == sizeof(*msg);
}

/* Socket daemon communication interface. */
static struct daemon_interface daemon_socket_interface = {
	.open  = daemon_open,
	.close = daemon_close,
	.read  = daemon_read,
	.write = daemon_write,
};

int register_daemon_interface(struct lib_context *lc)
{
	if (strcmp(lc->daemon_interface_name, "socket"))
		return 0;

	lc->daemon = &daemon_socket_interface;

	return 1;
}

void unregister_daemon_interface(struct lib_context *lc)
{
	lc->daemon = NULL;
}
