/*
 * Copyright (C) 2004,2005  Heinz Mauelshagen, Red Hat GmbH.
 *                          All rights reserved.
 *
 * See file LICENSE at the top of this source tree for license information.
 */

#include <errno.h>
#include "internal.h"

/* Create directory recusively. */
static int mk_dir_recursive(struct lib_context *lc, const char *dir)
{
	int ret = 1;
        char *orig, *s;

        if (!(orig = s = dbg_strdup((char*) dir)))
		return log_alloc_err(lc, __func__);

        /* Create parent directories */
        log_notice(lc, "creating directory %s", dir);
	do {
		s = remove_delimiter(s + 1, '/');
		if (mkdir(orig, 0777) && errno != EEXIST) {
			ret = 0;
			log_err(lc, "mkdir %s", orig);
			break;
		}

		add_delimiter(&s, '/');
	} while (s);

	dbg_free(orig);

        return ret;
}

/* Create directory. */
int mk_dir(struct lib_context *lc, const char *dir)
{
	struct stat info;

	/* If it doesn't exist yet, make it. */
	if (stat(dir, &info))
		return mk_dir_recursive(lc, dir);

	if (S_ISDIR(info.st_mode))
		return 1;

	LOG_ERR(lc, 0, "directory %s not found", dir);
}

static int rw_file(struct lib_context *lc, const char *who, int flags,
		   char *path, void *buffer, size_t size, loff_t offset)
{
	int fd, ret = 0;
	loff_t o;

	if ((fd = open(path, flags, lc->mode)) == -1)
		LOG_ERR(lc, 0, "opening \"%s\"\n", path);

#ifndef __KLIBC__
	if (offset && (o = lseek64(fd, offset, SEEK_SET)) == (loff_t) -1) {
#else
	if (offset && (o = lseek(fd, offset, SEEK_SET)) == (loff_t) -1) {
#endif
		log_err(lc, "%s: seeking device \"%s\" to %" PRIu64,
			who, path, offset);
		goto err;
	}

	if ((ret = ((flags & O_WRONLY) ? write(fd, buffer, size) :
					 read(fd, buffer, size)) != size)) {
		log_err(lc, "%s: %sing %s[%s]", who,
			(flags & O_WRONLY) ? "writ" : "read", path,
			strerror(errno));
		ret = 0;
		goto err;
	}

	ret = 1;

  err:
	close(fd);

	return ret;
}

int read_file(struct lib_context *lc, const char *who, char *path,
	      void *buffer, size_t size, loff_t offset)
{
	return rw_file(lc, who, O_RDONLY, path, buffer, size, offset);
}

int write_file(struct lib_context *lc, const char *who, char *path,
	       void *buffer, size_t size, loff_t offset)
{
	/* O_CREAT|O_TRUNC are noops on a devnode. */
	return rw_file(lc, who, O_WRONLY|O_CREAT|O_TRUNC, path,
		       buffer, size, offset);
}
