/*
 * Copyright (C) 2004,2005  Heinz Mauelshagen, Red Hat GmbH.
 *                          All rights reserved.
 *
 * See file LICENSE at the top of this source tree for license information.
 */

/*
 * Tool library
 */

#include <dmraid.h>
#include <ctype.h>

#ifndef __KLIBC__
# include <getopt.h>
#endif

#include <stdlib.h>
#include <string.h>
#include "commands.h"
#include "toollib.h"

/* [De]activate a RAID set. */
static int _change_set(struct lib_context *lc, void *rs, int arg)
{
	if (change_set(lc, (ACTIVATE & action) ? A_ACTIVATE : A_DEACTIVATE,
		       rs)) {
		log_info(lc, "%sctivating %s RAID set \"%s\"",
			 action & ACTIVATE ? "A": "Dea",
			 get_set_type(lc, rs), get_set_name(lc, rs));
		return 1;
	}

	return 0;
}

/* [De]activate RAID sets. */
/* FIXME: remove partition code in favour of kpartx ? */
static void process_partitions(struct lib_context *lc)
{
	discover_partitions(lc);
	process_sets(lc, _change_set, 0, PARTITIONS);
}

int activate_or_deactivate_sets(struct lib_context *lc, int arg)
{
	/* Discover partitions to deactivate RAID sets for and work on them. */
	if (DEACTIVATE & action)
		process_partitions(lc);

	process_sets(lc, _change_set, arg, SETS);

	/* Discover partitions to activate RAID sets for and work on them. */
	if ((ACTIVATE & action) && !(NOPARTITIONS & action))
		process_partitions(lc);

	return 1;
}

/* Build all sets or the ones given. */
void build_sets(struct lib_context *lc, char **sets)
{
	int o = 0;

	do {
		if (!group_set(lc, sets[o]))
			log_err(lc, "building set");

		if (!sets[o])
			break;
	} while (sets[++o]);
}

/* Convert a character string to lower case. */
void str_tolower(char *s)
{
	for (; *s; s++)
		*s = tolower(*s);
}

/*
 * Check if selected or all formats shall be used to read the metadata.
*/
void free_format_names(struct lib_context *lc)
{
	if (format_names) {
		char **p = format_names;

		while (*p)
			dbg_free(*p++);

		dbg_free(format_names);
		format_names = NULL;
	}
}

static int push_format_name(struct lib_context *lc, char ***fmt_names,
			    char *name)
{
	unsigned int i;
	char **p = *fmt_names;

	for (i = 0; p && p[i]; i++);
	if (!(p = dbg_realloc(p, (i + 2) * sizeof (*p)))) {
		free_format_names(lc);
		return 0;
	}

	p[i] = name;
	p[i+1] = NULL;
	*fmt_names = p;

	return 1;
}

/* Collapse a delimiter into one. */
static char *collapse_delimiter(struct lib_context *lc, char *str,
				size_t size, const char d)
{
	size_t i;
	char *p = str, *p1 = p;

	while ((p = strchr(p, d))) {
		/* "Remove" leading delimiter. */
		if (p == str) {
			str++;
			size--;
			continue;
		}

		for (p1 = p + 1; *p1 == d; p1++);
		if (++p < p1) {
			memmove(p, p1, (i = str + size - p));
			p[i] = 0;
			p1 = p - 1;
		}
	}

	/* Remove trailing delimiter. */
	if (*p1 == d)
		*p1 = 0;

	return str;
}

static int _valid_format(struct lib_context *lc, char *p,
			 char ***fmt_names, char **invalid)
{
	int ret;
	char *name;

	/* Multiple delimiters ? */
	if (!strlen(p))
		return 1;

	if (!(name = dbg_strdup(p)))
		return log_alloc_err(lc, __func__);
		
	if (!(ret = check_valid_format(lc, name)))
		*invalid = name;

	/*
	 * Even push an invalid format name,
	 * so that it will get freed on cleanup.
	 */
	return ret ? push_format_name(lc, fmt_names, name) : ret;
}

int valid_format(struct lib_context *lc, char *fmt,
		 char ***fmt_names, char **invalid)
{
	int ret = 1;
	char *p, *sep;
	const char d = ',';

	sep = remove_white_space(lc, fmt, strlen(fmt));
	sep = collapse_delimiter(lc, sep, strlen(sep), d);

	do {
		sep = remove_delimiter((p = sep), d);
		log_notice(lc, "checking format identifier %s", p);

		/* Ignore sequences of delimiters in format identifier list. */
		if (!(ret = _valid_format(lc, p, fmt_names, invalid)))
			break;

		add_delimiter(&sep, d);
	} while (sep);

	return ret;
}

void format_error(struct lib_context *lc, char **fmt_names, const char *error)
{
	if (fmt_names) {
		unsigned int i;

		log_print_nnl(lc, "No RAID %s with format: ", error);
		for (i = 0; fmt_names[i]; i++) {
			log_print_nnl(lc, "%s", fmt_names[i]);
			if (fmt_names[i+1])
				log_print_nnl(lc, ",");
		}

		log_print_nnl(lc, "\n");
	} else
		log_print(lc, "No Software RAID %ss", error);
}

/*
 * Toolib init/exit.
 */
int toollib_init(struct lib_context *lc)
{
	format_names = NULL;

	return 1;
}

void toollib_exit(struct lib_context *lc)
{
	free_format_names(lc);
}
